package MyWebApp.Servlets;

import MyWebApp.Beans.RequestHistoryBean;
import MyWebApp.Interfaces.IRequestHistory;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.charset.Charset;

public class AreaFormServlet extends HttpServlet {

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        resp.setContentType("text/html");
        resp.setCharacterEncoding("utf-8");

        try {
            resp.getOutputStream().write(generateHTML(req));
            resp.getOutputStream().flush();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }

    byte[] generateHTML(HttpServletRequest request) {
        byte[] res = null;
        float x=-6,y=-6,r=-6;
        boolean isValid = true;
        InitialContext ic;
        IRequestHistory hist_obj = null;
/*
        try
        {
            ic = new InitialContext();
            hist_obj = (IRequestHistory)ic.lookup("java:global/Lab8/RequestHistoryBean");
        }
        catch (NamingException ex)
        {
            ex.printStackTrace();
        }*/

        hist_obj = (IRequestHistory)request.getSession().getAttribute("HistoryBean");

        try {
            x = Float.parseFloat(request.getParameter("x_pos"));
            y = Float.parseFloat(request.getParameter("y_pos"));
            r = Float.parseFloat(request.getParameter("radius"));
        } catch (NumberFormatException ex) {
            isValid = false;
        }

        if(isValid && validateData(x,y,r))
        {
            boolean result = inRegion(x,y,r);
            res = genSuccPage(x,y,r, result);
            hist_obj.addEntry(x,y,r,result);
        }
        else
        {
            res = genErrorPage();
        }

        return res;
    }

    boolean validateData(float x, float y, float r) {
        boolean res = true;

        if(x > 5 || x < -5)
            res = false;


        return res;
    }

    byte[] genErrorPage()
    {
        byte[] ret = null;

        try
        {
/*
            ret = Files.readAllBytes(Paths.get(this.getClass().getResource("/templates/errTemplate.html").toURI()));
*/
            FileInputStream f_in = new FileInputStream(this.getClass().getResource("/templates/errTemplate.html").getFile());
            ByteArrayOutputStream buff = new ByteArrayOutputStream();

            int c;

            while((c = f_in.read()) != -1)
                buff.write(c);

            ret = buff.toByteArray();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }

        return ret;
    }

    byte[] genSuccPage(float x, float y, float r, boolean isInRegion)
    {
        byte[] res = null;
        try
        {
/*
            String myHTML = new String(Files.readAllBytes(Paths.get(this.getClass().getResource("/templates/succTemplate.html").toURI())), StandardCharsets.UTF_8);
*/
            FileInputStream f_in = new FileInputStream(this.getClass().getResource("/templates/succTemplate.html").getFile());
            ByteArrayOutputStream buff = new ByteArrayOutputStream();

            int c;

            while((c = f_in.read()) != -1)
                buff.write(c);

            String myHTML = new String(buff.toByteArray(), Charset.forName("utf-8"));
            String replaceString;
            if(isInRegion)
                replaceString = "Точка ("+ x +", "+ y + ")" +" в области <br/>" + "Радиус " + r;
            else
                replaceString = "Точка ("+ x +", "+ y + ")" +" вне области <br />" + "Радиус " + r;

            myHTML = myHTML.replaceAll("(.*)__RESULT_PLACE(.*)", replaceString);

            res = myHTML.getBytes();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        return  res;
    }

    boolean inRegion(float x, float y,float r)
    {
        boolean res = false;

        if(x >= 0)
        {
            if(y >= 0 )
                res = FirstQuarter(x,y,r);
        }
        else
        {
            if(y >= 0)
                res = SecondQuarter(x,y,r);
            else
                res = ThirdQuarter(x,y,r);
        }

        return res;
    }

    boolean FirstQuarter(float x, float y, float r)
    {
        boolean ret = true;

        if(Math.pow(x,2) + Math.pow(y,2) > Math.pow(r,2))
            ret = false;

        return ret;
    }

    boolean SecondQuarter(float x, float y, float r)
    {
        boolean ret = true;

        if(y > x + r)
            ret = false;

        return ret;
    }

    boolean ThirdQuarter(float x, float y, float r)
    {
        boolean ret = true;
        if(x < -(r/2) || y < -r)
            ret = false;

        return ret;
    }
}