package RegLocation.Client;

import RegLocation.Lib.Mark;
import RegLocation.Lib.MarkStates;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.net.SocketAddress;
import java.util.HashMap;
import java.util.Vector;

public class SilhouetteComponent extends JComponent
{
    final static long delay = 100; //ms
    static float R;

    Vector<MarksListener> MarksListeners;
    Vector<Mark> MAL;
    HashMap<Integer, MarkStates> Colors;

    SilhouetteView silhouetteView;
    SizesConverter converter;
    MarkProcessingPool pool;

    public SilhouetteComponent(SocketAddress addr, float R)
    {
        this.R = R;
        MarksListeners = new Vector<MarksListener>();
        MAL = new Vector<Mark>();
        converter = new SizesConverter(getSize());
        Colors = new HashMap<Integer, MarkStates>();
        silhouetteView = new SilhouetteView(MAL,converter,Colors);
        pool = new MarkProcessingPool(Colors, MAL, addr);


        Thread Animator = new Thread(new Runnable()
        {
            @Override
            public void run()
            {
                try
                {
                    while(true)
                    {
                        silhouetteView.paintPoints(getGraphics());
                        Thread.currentThread().sleep(delay);
                    }
                }
                catch (InterruptedException ex)
                {
                    Thread.currentThread().interrupt();
                }
            }
        });

        Animator.start();

        addMouseListener(new MouseAdapter()
        {
            @Override
            public void mouseClicked(MouseEvent e)
            {
                int x = e.getX();
                int y = e.getY();

                markAdd(converter.toFloat(x - getWidth()/2),converter.toFloat(getHeight()/2 - y));
            }
        });

        addComponentListener(new ComponentAdapter()
        {
            @Override
            public void componentResized(ComponentEvent e)
            {
                Rectangle b = e.getComponent().getBounds();
                e.getComponent().setBounds(b.x,b.y,b.width,b.width);
                converter.setSCS(e.getComponent().getSize());
                paintComponent(getGraphics());
                //Update component?
            }
        });

    }

    public void setR(float R)
    {
        SilhouetteComponent.R = R;
        ColorsReset();
        silhouetteView.paint(getGraphics());
    }

    void ColorsReset()
    {
        for(int code : Colors.keySet())
        {
            Colors.put(code, MarkStates.DontKnown);
            pool.addMarkInPool(code);
        }
    }

    public void addMarksListener(MarksListener listener)
    {
        MarksListeners.add(listener);
    }

    public void MarksActionPerformed(Mark mk)
    {
        for(MarksListener item : MarksListeners)
            item.actionPerformed(mk);
    }

    public void markAdd(float x,float y)
    {
        TOL type = TOL.FOREVER;

    /*  if(silhouette.InRegion(x,y) != 0)
        {
            type = TOL.EXPAND;
        }
    */
        Mark newMark = new Mark(x,y,type);
        int hashcode = newMark.hashCode();

        MAL.add(newMark);
        Colors.put(hashcode,MarkStates.DontKnown);
        pool.addMarkInPool(hashcode);

        MarksActionPerformed(newMark);
    }

    @Override
    public void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        silhouetteView.paint(g);
    }

    @Override
    public void update(Graphics g)
    {
        super.update(g);
        paintComponent(g);
    }


}
